@@ -1,12 +1,12 @@
 /* Support for printing Pascal values for GDB, the GNU debugger.
 
    Modified by Linaro.
-   Copyright (C) March 2023-2025 Linaro Limited (or its affiliates). All rights reserved.
+   Copyright (C) March 2023- Linaro Limited (or its affiliates). All rights reserved.
    
    Modified by Arm.
    Copyright (C) 1995-2023 Arm Limited (or its affiliates). All rights reserved.
    
-   Copyright (C) 2000-2023 Free Software Foundation, Inc.
+   Copyright (C) 2000-2013 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -26,7 +26,7 @@
 /* This file is derived from c-valprint.c */
 
 #include "defs.h"
-#include "gdbsupport/gdb_obstack.h"
+#include "gdb_obstack.h"
 #include "symtab.h"
 #include "gdbtypes.h"
 #include "expression.h"
@@ -43,16 +43,9 @@
 #include "p-lang.h"
 #include "cp-abi.h"
 #include "cp-support.h"
-#include "objfiles.h"
-#include "gdbsupport/byte-vector.h"
-#include "cli/cli-style.h"
+#include "exceptions.h"
 
 
-static void pascal_object_print_value_fields (struct value *, struct ui_file *,
-					      int,
-					      const struct value_print_options *,
-					      struct type **, int);
-
 /* Decorations for Pascal.  */
 
 static const struct generic_val_print_decorations p_decorations =
@@ -62,98 +55,103 @@ static const struct generic_val_print_decorations p_decorations =
   " * I",
   "true",
   "false",
-  "void",
-  "{",
-  "}"
+  "void"
 };
 
-/* See p-lang.h.  */
+/* See val_print for a description of the various parameters of this
+   function; they are identical.  */
 
 void
-pascal_language::value_print_inner (struct value *val,
-				    struct ui_file *stream, int recurse,
-				    const struct value_print_options *options) const
-
+pascal_val_print (struct type *type, const gdb_byte *valaddr,
+		  int embedded_offset, CORE_ADDR address,
+		  struct ui_file *stream, int recurse,
+		  const struct value *original_value,
+		  const struct value_print_options *options)
 {
-  struct type *type = check_typedef (value_type (val));
-  struct gdbarch *gdbarch = type->arch ();
-  enum bfd_endian byte_order = type_byte_order (type);
+  struct gdbarch *gdbarch = get_type_arch (type);
+  enum bfd_endian byte_order = gdbarch_byte_order (gdbarch);
   unsigned int i = 0;	/* Number of characters printed */
   unsigned len;
+  LONGEST low_bound, high_bound;
   struct type *elttype;
   unsigned eltlen;
   int length_pos, length_size, string_pos;
   struct type *char_type;
   CORE_ADDR addr;
   int want_space = 0;
-  const gdb_byte *valaddr = value_contents_for_printing (val).data ();
 
-  switch (type->code ())
+  CHECK_TYPEDEF (type);
+  switch (TYPE_CODE (type))
     {
     case TYPE_CODE_ARRAY:
-      {
-	LONGEST low_bound, high_bound;
+      if (get_array_bounds (type, &low_bound, &high_bound))
+	{
+	  len = high_bound - low_bound + 1;
+	  elttype = check_typedef (TYPE_TARGET_TYPE (type));
+	  eltlen = TYPE_LENGTH (elttype);
+	  if (options->prettyprint_arrays)
+	    {
+	      print_spaces_filtered (2 + 2 * recurse, stream);
+	    }
+	  /* If 's' format is used, try to print out as string.
+	     If no format is given, print as string if element type
+	     is of TYPE_CODE_CHAR and element size is 1,2 or 4.  */
+	  if (options->format == 's'
+	      || ((eltlen == 1 || eltlen == 2 || eltlen == 4)
+		  && TYPE_CODE (elttype) == TYPE_CODE_CHAR
+		  && options->format == 0))
+	    {
+	      /* If requested, look for the first null char and only print
+	         elements up to it.  */
+	      if (options->stop_print_at_null)
+		{
+		  unsigned int temp_len;
 
-	if (get_array_bounds (type, &low_bound, &high_bound))
-	  {
-	    len = high_bound - low_bound + 1;
-	    elttype = check_typedef (type->target_type ());
-	    eltlen = elttype->length ();
-	    /* If 's' format is used, try to print out as string.
-	       If no format is given, print as string if element type
-	       is of TYPE_CODE_CHAR and element size is 1,2 or 4.  */
-	    if (options->format == 's'
-		|| ((eltlen == 1 || eltlen == 2 || eltlen == 4)
-		    && elttype->code () == TYPE_CODE_CHAR
-		    && options->format == 0))
-	      {
-		/* If requested, look for the first null char and only print
-		   elements up to it.  */
-		if (options->stop_print_at_null)
-		  {
-		    unsigned int temp_len;
+		  /* Look for a NULL char.  */
+		  for (temp_len = 0;
+		       extract_unsigned_integer (valaddr + embedded_offset +
+						 temp_len * eltlen, eltlen,
+						 byte_order)
+		       && temp_len < len && temp_len < options->print_smax;
+		       temp_len++);
+		  len = temp_len;
+		}
 
-		    /* Look for a NULL char.  */
-		    for (temp_len = 0;
-			 extract_unsigned_integer (valaddr + temp_len * eltlen,
-						   eltlen, byte_order)
-			   && temp_len < len && temp_len < options->print_max;
-			 temp_len++);
-		    len = temp_len;
-		  }
-
-		printstr (stream, type->target_type (), valaddr, len,
-			  NULL, 0, options);
-		i = len;
-	      }
-	    else
-	      {
-		gdb_printf (stream, "{");
-		/* If this is a virtual function table, print the 0th
-		   entry specially, and the rest of the members normally.  */
-		if (pascal_object_is_vtbl_ptr_type (elttype))
-		  {
-		    i = 1;
-		    gdb_printf (stream, "%d vtable entries", len - 1);
-		  }
-		else
-		  {
-		    i = 0;
-		  }
-		value_print_array_elements (val, stream, recurse, options, i);
-		gdb_printf (stream, "}");
-	      }
-	    break;
-	  }
-	/* Array of unspecified length: treat like pointer to first elt.  */
-	addr = value_address (val);
-      }
+	      LA_PRINT_STRING (stream, TYPE_TARGET_TYPE (type),
+			       valaddr + embedded_offset, len, NULL, 0,
+			       options);
+	      i = len;
+	    }
+	  else
+	    {
+	      fprintf_filtered (stream, "{");
+	      /* If this is a virtual function table, print the 0th
+	         entry specially, and the rest of the members normally.  */
+	      if (pascal_object_is_vtbl_ptr_type (elttype))
+		{
+		  i = 1;
+		  fprintf_filtered (stream, "%d vtable entries", len - 1);
+		}
+	      else
+		{
+		  i = 0;
+		}
+	      val_print_array_elements (type, valaddr, embedded_offset,
+					address, stream, recurse,
+					original_value, options, i);
+	      fprintf_filtered (stream, "}");
+	    }
+	  break;
+	}
+      /* Array of unspecified length: treat like pointer to first elt.  */
+      addr = address + embedded_offset;
       goto print_unpacked_pointer;
 
     case TYPE_CODE_PTR:
       if (options->format && options->format != 's')
 	{
-	  value_print_scalar_formatted (val, options, 0, stream);
+	  val_print_scalar_formatted (type, valaddr, embedded_offset,
+				      original_value, options, 0, stream);
 	  break;
 	}
       if (options->vtblprint && pascal_object_is_vtbl_ptr_type (type))
@@ -162,18 +160,18 @@ pascal_language::value_print_inner (struct value *val,
 	  /* Print vtable entry - we only get here if we ARE using
 	     -fvtable_thunks.  (Otherwise, look under TYPE_CODE_STRUCT.)  */
 	  /* Extract the address, assume that it is unsigned.  */
-	  addr = extract_unsigned_integer (valaddr,
-					   type->length (), byte_order);
+	  addr = extract_unsigned_integer (valaddr + embedded_offset,
+					   TYPE_LENGTH (type), byte_order);
 	  print_address_demangle (options, gdbarch, addr, stream, demangle);
 	  break;
 	}
-      check_typedef (type->target_type ());
+      check_typedef (TYPE_TARGET_TYPE (type));
 
-      addr = unpack_pointer (type, valaddr);
+      addr = unpack_pointer (type, valaddr + embedded_offset);
     print_unpacked_pointer:
-      elttype = check_typedef (type->target_type ());
+      elttype = check_typedef (TYPE_TARGET_TYPE (type));
 
-      if (elttype->code () == TYPE_CODE_FUNC)
+      if (TYPE_CODE (elttype) == TYPE_CODE_FUNC)
 	{
 	  /* Try to print what function it points to.  */
 	  print_address_demangle (options, gdbarch, addr, stream, demangle);
@@ -182,22 +180,22 @@ pascal_language::value_print_inner (struct value *val,
 
       if (options->addressprint && options->format != 's')
 	{
-	  gdb_puts (paddress (gdbarch, addr), stream);
+	  fputs_filtered (paddress (gdbarch, addr), stream);
 	  want_space = 1;
 	}
 
       /* For a pointer to char or unsigned char, also print the string
 	 pointed to, unless pointer is null.  */
-      if (((elttype->length () == 1
-	   && (elttype->code () == TYPE_CODE_INT
-	       || elttype->code () == TYPE_CODE_CHAR))
-	   || ((elttype->length () == 2 || elttype->length () == 4)
-	       && elttype->code () == TYPE_CODE_CHAR))
+      if (((TYPE_LENGTH (elttype) == 1
+	   && (TYPE_CODE (elttype) == TYPE_CODE_INT
+	      || TYPE_CODE (elttype) == TYPE_CODE_CHAR))
+	  || ((TYPE_LENGTH (elttype) == 2 || TYPE_LENGTH (elttype) == 4)
+	      && TYPE_CODE (elttype) == TYPE_CODE_CHAR))
 	  && (options->format == 0 || options->format == 's')
 	  && addr != 0)
 	{
 	  if (want_space)
-	    gdb_puts (" ", stream);
+	    fputs_filtered (" ", stream);
 	  /* No wide string yet.  */
 	  i = val_print_string (elttype, NULL, addr, -1, stream, options);
 	}
@@ -206,16 +204,16 @@ pascal_language::value_print_inner (struct value *val,
 	 as GDB does not recognize stabs pascal strings
 	 Pascal strings are mapped to records
 	 with lowercase names PM.  */
-      if (pascal_is_string_type (elttype, &length_pos, &length_size,
-				 &string_pos, &char_type, NULL) > 0
+      if (is_pascal_string_type (elttype, &length_pos, &length_size,
+				 &string_pos, &char_type, NULL)
 	  && addr != 0)
 	{
 	  ULONGEST string_length;
-	  gdb_byte *buffer;
+	  void *buffer;
 
 	  if (want_space)
-	    gdb_puts (" ", stream);
-	  buffer = (gdb_byte *) xmalloc (length_size);
+	    fputs_filtered (" ", stream);
+	  buffer = xmalloc (length_size);
 	  read_memory (addr + length_pos, buffer, length_size);
 	  string_length = extract_unsigned_integer (buffer, length_size,
 						    byte_order);
@@ -227,53 +225,53 @@ pascal_language::value_print_inner (struct value *val,
       else if (pascal_object_is_vtbl_member (type))
 	{
 	  /* Print vtbl's nicely.  */
-	  CORE_ADDR vt_address = unpack_pointer (type, valaddr);
-	  struct bound_minimal_symbol msymbol =
+	  CORE_ADDR vt_address = unpack_pointer (type,
+						 valaddr + embedded_offset);
+	  struct minimal_symbol *msymbol =
 	    lookup_minimal_symbol_by_pc (vt_address);
 
 	  /* If 'symbol_print' is set, we did the work above.  */
 	  if (!options->symbol_print
-	      && (msymbol.minsym != NULL)
-	      && (vt_address == msymbol.value_address ()))
+	      && (msymbol != NULL)
+	      && (vt_address == SYMBOL_VALUE_ADDRESS (msymbol)))
 	    {
 	      if (want_space)
-		gdb_puts (" ", stream);
-	      gdb_puts ("<", stream);
-	      gdb_puts (msymbol.minsym->print_name (), stream);
-	      gdb_puts (">", stream);
+		fputs_filtered (" ", stream);
+	      fputs_filtered ("<", stream);
+	      fputs_filtered (SYMBOL_PRINT_NAME (msymbol), stream);
+	      fputs_filtered (">", stream);
 	      want_space = 1;
 	    }
 	  if (vt_address && options->vtblprint)
 	    {
 	      struct value *vt_val;
-	      struct symbol *wsym = NULL;
+	      struct symbol *wsym = (struct symbol *) NULL;
 	      struct type *wtype;
+	      struct block *block = (struct block *) NULL;
+	      struct field_of_this_result is_this_fld;
 
 	      if (want_space)
-		gdb_puts (" ", stream);
+		fputs_filtered (" ", stream);
 
-	      if (msymbol.minsym != NULL)
-		{
-		  const char *search_name = msymbol.minsym->search_name ();
-		  wsym = lookup_symbol_search_name (search_name, NULL,
-						    VAR_DOMAIN).symbol;
-		}
+	      if (msymbol != NULL)
+		wsym = lookup_symbol (SYMBOL_LINKAGE_NAME (msymbol), block,
+				      VAR_DOMAIN, &is_this_fld);
 
 	      if (wsym)
 		{
-		  wtype = wsym->type ();
+		  wtype = SYMBOL_TYPE (wsym);
 		}
 	      else
 		{
-		  wtype = type->target_type ();
+		  wtype = TYPE_TARGET_TYPE (type);
 		}
 	      vt_val = value_at (wtype, vt_address);
 	      common_val_print (vt_val, stream, recurse + 1, options,
 				current_language);
-	      if (options->prettyformat)
+	      if (options->pretty)
 		{
-		  gdb_printf (stream, "\n");
-		  print_spaces (2 + 2 * recurse, stream);
+		  fprintf_filtered (stream, "\n");
+		  print_spaces_filtered (2 + 2 * recurse, stream);
 		}
 	    }
 	}
@@ -292,13 +290,15 @@ pascal_language::value_print_inner (struct value *val,
     case TYPE_CODE_UNDEF:
     case TYPE_CODE_BOOL:
     case TYPE_CODE_CHAR:
-      generic_value_print (val, stream, recurse, options, &p_decorations);
+      generic_val_print (type, valaddr, embedded_offset, address,
+			 stream, recurse, original_value, options,
+			 &p_decorations);
       break;
 
     case TYPE_CODE_UNION:
       if (recurse && !options->unionprint)
 	{
-	  gdb_printf (stream, "{...}");
+	  fprintf_filtered (stream, "{...}");
 	  break;
 	}
       /* Fall through.  */
@@ -311,64 +311,72 @@ pascal_language::value_print_inner (struct value *val,
 	  /* Extract the address, assume that it is unsigned.  */
 	  print_address_demangle
 	    (options, gdbarch,
-	     extract_unsigned_integer
-	       (valaddr + type->field (VTBL_FNADDR_OFFSET).loc_bitpos () / 8,
-		type->field (VTBL_FNADDR_OFFSET).type ()->length (),
-		byte_order),
+	     extract_unsigned_integer (valaddr + embedded_offset
+				       + TYPE_FIELD_BITPOS (type,
+							    VTBL_FNADDR_OFFSET) / 8,
+				       TYPE_LENGTH (TYPE_FIELD_TYPE (type,
+								     VTBL_FNADDR_OFFSET)),
+				       byte_order),
 	     stream, demangle);
 	}
       else
 	{
-	  if (pascal_is_string_type (type, &length_pos, &length_size,
-				     &string_pos, &char_type, NULL) > 0)
+          if (is_pascal_string_type (type, &length_pos, &length_size,
+                                     &string_pos, &char_type, NULL))
 	    {
-	      len = extract_unsigned_integer (valaddr + length_pos,
-					      length_size, byte_order);
-	      printstr (stream, char_type, valaddr + string_pos, len,
-			NULL, 0, options);
+	      len = extract_unsigned_integer (valaddr + embedded_offset
+					      + length_pos, length_size,
+					      byte_order);
+	      LA_PRINT_STRING (stream, char_type,
+			       valaddr + embedded_offset + string_pos,
+			       len, NULL, 0, options);
 	    }
 	  else
-	    pascal_object_print_value_fields (val, stream, recurse,
-					      options, NULL, 0);
+	    pascal_object_print_value_fields (type, valaddr, embedded_offset,
+					      address, stream, recurse,
+					      original_value, options,
+					      NULL, 0);
 	}
       break;
 
     case TYPE_CODE_SET:
-      elttype = type->index_type ();
-      elttype = check_typedef (elttype);
-      if (elttype->is_stub ())
+      elttype = TYPE_INDEX_TYPE (type);
+      CHECK_TYPEDEF (elttype);
+      if (TYPE_STUB (elttype))
 	{
-	  fprintf_styled (stream, metadata_style.style (), "<incomplete type>");
+	  fprintf_filtered (stream, "<incomplete type>");
+	  gdb_flush (stream);
 	  break;
 	}
       else
 	{
 	  struct type *range = elttype;
 	  LONGEST low_bound, high_bound;
+	  int i;
 	  int need_comma = 0;
 
-	  gdb_puts ("[", stream);
+	  fputs_filtered ("[", stream);
 
-	  int bound_info = (get_discrete_bounds (range, &low_bound, &high_bound)
-			    ? 0 : -1);
-	  if (low_bound == 0 && high_bound == -1 && type->length () > 0)
+	  i = get_discrete_bounds (range, &low_bound, &high_bound);
+	  if (low_bound == 0 && high_bound == -1 && TYPE_LENGTH (type) > 0)
 	    {
 	      /* If we know the size of the set type, we can figure out the
 	      maximum value.  */
-	      bound_info = 0;
-	      high_bound = type->length () * TARGET_CHAR_BIT - 1;
-	      range->bounds ()->high.set_const_val (high_bound);
+	      i = 0;
+	      high_bound = TYPE_LENGTH (type) * TARGET_CHAR_BIT - 1;
+	      TYPE_HIGH_BOUND (range) = high_bound;
 	    }
 	maybe_bad_bstring:
-	  if (bound_info < 0)
+	  if (i < 0)
 	    {
-	      fputs_styled ("<error value>", metadata_style.style (), stream);
+	      fputs_filtered ("<error value>", stream);
 	      goto done;
 	    }
 
 	  for (i = low_bound; i <= high_bound; i++)
 	    {
-	      int element = value_bit_index (type, valaddr, i);
+	      int element = value_bit_index (type,
+					     valaddr + embedded_offset, i);
 
 	      if (element < 0)
 		{
@@ -378,38 +386,41 @@ pascal_language::value_print_inner (struct value *val,
 	      if (element)
 		{
 		  if (need_comma)
-		    gdb_puts (", ", stream);
+		    fputs_filtered (", ", stream);
 		  print_type_scalar (range, i, stream);
 		  need_comma = 1;
 
 		  if (i + 1 <= high_bound
-		      && value_bit_index (type, valaddr, ++i))
+		      && value_bit_index (type,
+					  valaddr + embedded_offset, ++i))
 		    {
 		      int j = i;
 
-		      gdb_puts ("..", stream);
+		      fputs_filtered ("..", stream);
 		      while (i + 1 <= high_bound
-			     && value_bit_index (type, valaddr, ++i))
+			     && value_bit_index (type,
+						 valaddr + embedded_offset,
+						 ++i))
 			j = i;
 		      print_type_scalar (range, j, stream);
 		    }
 		}
 	    }
 	done:
-	  gdb_puts ("]", stream);
+	  fputs_filtered ("]", stream);
 	}
       break;
 
     default:
       error (_("Invalid pascal type code %d in symbol table."),
-	     type->code ());
+	     TYPE_CODE (type));
     }
+  gdb_flush (stream);
 }
-
 
 void
-pascal_language::value_print (struct value *val, struct ui_file *stream,
-			      const struct value_print_options *options) const
+pascal_value_print (struct value *val, struct ui_file *stream,
+		    const struct value_print_options *options)
 {
   struct type *type = value_type (val);
   struct value_print_options opts = *options;
@@ -422,23 +433,23 @@ pascal_language::value_print (struct value *val, struct ui_file *stream,
 
      Object pascal: if it is a member pointer, we will take care
      of that when we print it.  */
-  if (type->code () == TYPE_CODE_PTR
-      || type->code () == TYPE_CODE_REF)
+  if (TYPE_CODE (type) == TYPE_CODE_PTR
+      || TYPE_CODE (type) == TYPE_CODE_REF)
     {
       /* Hack:  remove (char *) for char strings.  Their
-	 type is indicated by the quoted string anyway.  */
-      if (type->code () == TYPE_CODE_PTR
-	  && type->name () == NULL
-	  && type->target_type ()->name () != NULL
-	  && strcmp (type->target_type ()->name (), "char") == 0)
+         type is indicated by the quoted string anyway.  */
+      if (TYPE_CODE (type) == TYPE_CODE_PTR
+	  && TYPE_NAME (type) == NULL
+	  && TYPE_NAME (TYPE_TARGET_TYPE (type)) != NULL
+	  && strcmp (TYPE_NAME (TYPE_TARGET_TYPE (type)), "char") == 0)
 	{
 	  /* Print nothing.  */
 	}
       else
 	{
-	  gdb_printf (stream, "(");
+	  fprintf_filtered (stream, "(");
 	  type_print (type, "", stream, -1);
-	  gdb_printf (stream, ") ");
+	  fprintf_filtered (stream, ") ");
 	}
     }
   common_val_print (val, stream, 0, &opts, current_language);
@@ -449,8 +460,8 @@ static void
 show_pascal_static_field_print (struct ui_file *file, int from_tty,
 				struct cmd_list_element *c, const char *value)
 {
-  gdb_printf (file, _("Printing of pascal static members is %s.\n"),
-	      value);
+  fprintf_filtered (file, _("Printing of pascal static members is %s.\n"),
+		    value);
 }
 
 static struct obstack dont_print_vb_obstack;
@@ -460,7 +471,10 @@ static void pascal_object_print_static_field (struct value *,
 					      struct ui_file *, int,
 					      const struct value_print_options *);
 
-static void pascal_object_print_value (struct value *, struct ui_file *, int,
+static void pascal_object_print_value (struct type *, const gdb_byte *,
+				       int,
+				       CORE_ADDR, struct ui_file *, int,
+				       const struct value *,
 				       const struct value_print_options *,
 				       struct type **);
 
@@ -474,10 +488,10 @@ const char pascal_vtbl_ptr_name[] =
 int
 pascal_object_is_vtbl_ptr_type (struct type *type)
 {
-  const char *type_name = type->name ();
+  const char *typename = type_name_no_tag (type);
 
-  return (type_name != NULL
-	  && strcmp (type_name, pascal_vtbl_ptr_name) == 0);
+  return (typename != NULL
+	  && strcmp (typename, pascal_vtbl_ptr_name) == 0);
 }
 
 /* Return truth value for the assertion that TYPE is of the type
@@ -486,18 +500,18 @@ pascal_object_is_vtbl_ptr_type (struct type *type)
 int
 pascal_object_is_vtbl_member (struct type *type)
 {
-  if (type->code () == TYPE_CODE_PTR)
+  if (TYPE_CODE (type) == TYPE_CODE_PTR)
     {
-      type = type->target_type ();
-      if (type->code () == TYPE_CODE_ARRAY)
+      type = TYPE_TARGET_TYPE (type);
+      if (TYPE_CODE (type) == TYPE_CODE_ARRAY)
 	{
-	  type = type->target_type ();
-	  if (type->code () == TYPE_CODE_STRUCT	/* If not using
+	  type = TYPE_TARGET_TYPE (type);
+	  if (TYPE_CODE (type) == TYPE_CODE_STRUCT	/* If not using
 							   thunks.  */
-	      || type->code () == TYPE_CODE_PTR)	/* If using thunks.  */
+	      || TYPE_CODE (type) == TYPE_CODE_PTR)	/* If using thunks.  */
 	    {
 	      /* Virtual functions tables are full of pointers
-		 to virtual functions.  */
+	         to virtual functions.  */
 	      return pascal_object_is_vtbl_ptr_type (type);
 	    }
 	}
@@ -505,44 +519,48 @@ pascal_object_is_vtbl_member (struct type *type)
   return 0;
 }
 
-/* Helper function for print pascal objects.
+/* Mutually recursive subroutines of pascal_object_print_value and
+   c_val_print to print out a structure's fields:
+   pascal_object_print_value_fields and pascal_object_print_value.
 
-   VAL, STREAM, RECURSE, and OPTIONS have the same meanings as in
-   pascal_object_print_value and c_value_print.
+   TYPE, VALADDR, ADDRESS, STREAM, RECURSE, and OPTIONS have the
+   same meanings as in pascal_object_print_value and c_val_print.
 
    DONT_PRINT is an array of baseclass types that we
    should not print, or zero if called from top level.  */
 
-static void
-pascal_object_print_value_fields (struct value *val, struct ui_file *stream,
+void
+pascal_object_print_value_fields (struct type *type, const gdb_byte *valaddr,
+				  int offset,
+				  CORE_ADDR address, struct ui_file *stream,
 				  int recurse,
+				  const struct value *val,
 				  const struct value_print_options *options,
 				  struct type **dont_print_vb,
 				  int dont_print_statmem)
 {
   int i, len, n_baseclasses;
-  char *last_dont_print
-    = (char *) obstack_next_free (&dont_print_statmem_obstack);
+  char *last_dont_print = obstack_next_free (&dont_print_statmem_obstack);
 
-  struct type *type = check_typedef (value_type (val));
+  CHECK_TYPEDEF (type);
 
-  gdb_printf (stream, "{");
-  len = type->num_fields ();
+  fprintf_filtered (stream, "{");
+  len = TYPE_NFIELDS (type);
   n_baseclasses = TYPE_N_BASECLASSES (type);
 
   /* Print out baseclasses such that we don't print
      duplicates of virtual baseclasses.  */
   if (n_baseclasses > 0)
-    pascal_object_print_value (val, stream, recurse + 1,
+    pascal_object_print_value (type, valaddr, offset, address,
+			       stream, recurse + 1, val,
 			       options, dont_print_vb);
 
   if (!len && n_baseclasses == 1)
-    fprintf_styled (stream, metadata_style.style (), "<No data fields>");
+    fprintf_filtered (stream, "<No data fields>");
   else
     {
       struct obstack tmp_obstack = dont_print_statmem_obstack;
       int fields_seen = 0;
-      const gdb_byte *valaddr = value_contents_for_printing (val).data ();
 
       if (dont_print_statmem == 0)
 	{
@@ -556,74 +574,73 @@ pascal_object_print_value_fields (struct value *val, struct ui_file *stream,
 	{
 	  /* If requested, skip printing of static fields.  */
 	  if (!options->pascal_static_field_print
-	      && field_is_static (&type->field (i)))
+	      && field_is_static (&TYPE_FIELD (type, i)))
 	    continue;
 	  if (fields_seen)
-	    gdb_printf (stream, ", ");
+	    fprintf_filtered (stream, ", ");
 	  else if (n_baseclasses > 0)
 	    {
-	      if (options->prettyformat)
+	      if (options->pretty)
 		{
-		  gdb_printf (stream, "\n");
-		  print_spaces (2 + 2 * recurse, stream);
-		  gdb_puts ("members of ", stream);
-		  gdb_puts (type->name (), stream);
-		  gdb_puts (": ", stream);
+		  fprintf_filtered (stream, "\n");
+		  print_spaces_filtered (2 + 2 * recurse, stream);
+		  fputs_filtered ("members of ", stream);
+		  fputs_filtered (type_name_no_tag (type), stream);
+		  fputs_filtered (": ", stream);
 		}
 	    }
 	  fields_seen = 1;
 
-	  if (options->prettyformat)
+	  if (options->pretty)
 	    {
-	      gdb_printf (stream, "\n");
-	      print_spaces (2 + 2 * recurse, stream);
+	      fprintf_filtered (stream, "\n");
+	      print_spaces_filtered (2 + 2 * recurse, stream);
 	    }
 	  else
 	    {
-	      stream->wrap_here (2 + 2 * recurse);
+	      wrap_here (n_spaces (2 + 2 * recurse));
 	    }
 
-	  annotate_field_begin (type->field (i).type ());
+	  annotate_field_begin (TYPE_FIELD_TYPE (type, i));
 
-	  if (field_is_static (&type->field (i)))
-	    {
-	      gdb_puts ("static ", stream);
-	      fprintf_symbol (stream,
-			      type->field (i).name (),
-			      current_language->la_language,
-			      DMGL_PARAMS | DMGL_ANSI);
-	    }
-	  else
-	    fputs_styled (type->field (i).name (),
-			  variable_name_style.style (), stream);
+	  if (field_is_static (&TYPE_FIELD (type, i)))
+	    fputs_filtered ("static ", stream);
+	  fprintf_symbol_filtered (stream, TYPE_FIELD_NAME (type, i),
+				   language_cplus,
+				   DMGL_PARAMS | DMGL_ANSI);
 	  annotate_field_name_end ();
-	  gdb_puts (" = ", stream);
+	  fputs_filtered (" = ", stream);
 	  annotate_field_value ();
 
-	  if (!field_is_static (&type->field (i))
+	  if (!field_is_static (&TYPE_FIELD (type, i))
 	      && TYPE_FIELD_PACKED (type, i))
 	    {
 	      struct value *v;
 
 	      /* Bitfields require special handling, especially due to byte
-		 order problems.  */
+	         order problems.  */
 	      if (TYPE_FIELD_IGNORE (type, i))
 		{
-		  fputs_styled ("<optimized out or zero length>",
-				metadata_style.style (), stream);
+		  fputs_filtered ("<optimized out or zero length>", stream);
 		}
-	      else if (value_bits_synthetic_pointer
-			 (val, type->field (i).loc_bitpos (),
-			  TYPE_FIELD_BITSIZE (type, i)))
+	      else if (value_bits_synthetic_pointer (val,
+						     TYPE_FIELD_BITPOS (type,
+									i),
+						     TYPE_FIELD_BITSIZE (type,
+									 i)))
 		{
-		  fputs_styled (_("<synthetic pointer>"),
-				metadata_style.style (), stream);
+		  fputs_filtered (_("<synthetic pointer>"), stream);
+		}
+	      else if (!value_bits_valid (val, TYPE_FIELD_BITPOS (type, i),
+					  TYPE_FIELD_BITSIZE (type, i)))
+		{
+		  val_print_optimized_out (stream);
 		}
 	      else
 		{
 		  struct value_print_options opts = *options;
 
-		  v = value_field_bitfield (type, i, valaddr, 0, val);
+		  v = value_field_bitfield (type, i, valaddr, offset, val);
 
 		  opts.deref_ref = 0;
 		  common_val_print (v, stream, recurse + 1, &opts,
@@ -634,19 +651,18 @@ pascal_object_print_value_fields (struct value *val, struct ui_file *stream,
 	    {
 	      if (TYPE_FIELD_IGNORE (type, i))
 		{
-		  fputs_styled ("<optimized out or zero length>",
-				metadata_style.style (), stream);
+		  fputs_filtered ("<optimized out or zero length>", stream);
 		}
-	      else if (field_is_static (&type->field (i)))
+	      else if (field_is_static (&TYPE_FIELD (type, i)))
 		{
 		  /* struct value *v = value_static_field (type, i);
 		     v4.17 specific.  */
 		  struct value *v;
 
-		  v = value_field_bitfield (type, i, valaddr, 0, val);
+		  v = value_field_bitfield (type, i, valaddr, offset, val);
 
 		  if (v == NULL)
-		    val_print_optimized_out (NULL, stream);
+		    val_print_optimized_out (stream);
 		  else
 		    pascal_object_print_static_field (v, stream, recurse + 1,
 						      options);
@@ -656,11 +672,14 @@ pascal_object_print_value_fields (struct value *val, struct ui_file *stream,
 		  struct value_print_options opts = *options;
 
 		  opts.deref_ref = 0;
-
-		  struct value *v = value_primitive_field (val, 0, i,
-							   value_type (val));
-		  common_val_print (v, stream, recurse + 1, &opts,
-				    current_language);
+		  /* val_print (TYPE_FIELD_TYPE (type, i),
+		     valaddr + TYPE_FIELD_BITPOS (type, i) / 8,
+		     address + TYPE_FIELD_BITPOS (type, i) / 8, 0,
+		     stream, format, 0, recurse + 1, pretty); */
+		  val_print (TYPE_FIELD_TYPE (type, i),
+			     valaddr, offset + TYPE_FIELD_BITPOS (type, i) / 8,
+			     address, stream, recurse + 1, val, &opts,
+			     current_language);
 		}
 	    }
 	  annotate_field_end ();
@@ -674,44 +693,49 @@ pascal_object_print_value_fields (struct value *val, struct ui_file *stream,
 	  dont_print_statmem_obstack = tmp_obstack;
 	}
 
-      if (options->prettyformat)
+      if (options->pretty)
 	{
-	  gdb_printf (stream, "\n");
-	  print_spaces (2 * recurse, stream);
+	  fprintf_filtered (stream, "\n");
+	  print_spaces_filtered (2 * recurse, stream);
 	}
     }
-  gdb_printf (stream, "}");
+  fprintf_filtered (stream, "}");
 }
 
 /* Special val_print routine to avoid printing multiple copies of virtual
    baseclasses.  */
 
 static void
-pascal_object_print_value (struct value *val, struct ui_file *stream,
+pascal_object_print_value (struct type *type, const gdb_byte *valaddr,
+			   int offset,
+			   CORE_ADDR address, struct ui_file *stream,
 			   int recurse,
+			   const struct value *val,
 			   const struct value_print_options *options,
 			   struct type **dont_print_vb)
 {
   struct type **last_dont_print
     = (struct type **) obstack_next_free (&dont_print_vb_obstack);
   struct obstack tmp_obstack = dont_print_vb_obstack;
-  struct type *type = check_typedef (value_type (val));
   int i, n_baseclasses = TYPE_N_BASECLASSES (type);
 
   if (dont_print_vb == 0)
     {
       /* If we're at top level, carve out a completely fresh
-	 chunk of the obstack and use that until this particular
-	 invocation returns.  */
+         chunk of the obstack and use that until this particular
+         invocation returns.  */
       /* Bump up the high-water mark.  Now alpha is omega.  */
       obstack_finish (&dont_print_vb_obstack);
     }
 
   for (i = 0; i < n_baseclasses; i++)
     {
-      LONGEST boffset = 0;
+      int boffset = 0;
       struct type *baseclass = check_typedef (TYPE_BASECLASS (type, i));
-      const char *basename = baseclass->name ();
+      const char *basename = type_name_no_tag (baseclass);
+      const gdb_byte *base_valaddr = NULL;
+      int thisoffset;
+      volatile struct gdb_exception ex;
       int skip = 0;
 
       if (BASETYPE_VIA_VIRTUAL (type, i))
@@ -729,65 +753,68 @@ pascal_object_print_value (struct value *val, struct ui_file *stream,
 	  obstack_ptr_grow (&dont_print_vb_obstack, baseclass);
 	}
 
-      struct value *base_value;
-      try
-	{
-	  base_value = value_primitive_field (val, 0, i, type);
-	}
-      catch (const gdb_exception_error &ex)
-	{
-	  base_value = nullptr;
-	  if (ex.error == NOT_AVAILABLE_ERROR)
-	    skip = -1;
-	  else
-	    skip = 1;
-	}
+      thisoffset = offset;
 
-      if (skip == 0)
+      TRY_CATCH (ex, RETURN_MASK_ERROR)
 	{
+	  boffset = baseclass_offset (type, i, valaddr, offset, address, val);
+	}
+      if (ex.reason < 0 && ex.error == NOT_AVAILABLE_ERROR)
+	skip = -1;
+      else if (ex.reason < 0)
+	skip = 1;
+      else
+	{
+	  skip = 0;
+
 	  /* The virtual base class pointer might have been clobbered by the
 	     user program. Make sure that it still points to a valid memory
 	     location.  */
 
-	  if (boffset < 0 || boffset >= type->length ())
+	  if (boffset < 0 || boffset >= TYPE_LENGTH (type))
 	    {
-	      CORE_ADDR address= value_address (val);
-	      gdb::byte_vector buf (baseclass->length ());
+	      gdb_byte *buf;
+	      struct cleanup *back_to;
 
-	      if (target_read_memory (address + boffset, buf.data (),
-				      baseclass->length ()) != 0)
+	      buf = xmalloc (TYPE_LENGTH (baseclass));
+	      back_to = make_cleanup (xfree, buf);
+
+	      base_valaddr = buf;
+	      if (target_read_memory (address + boffset, buf,
+				      TYPE_LENGTH (baseclass)) != 0)
 		skip = 1;
-	      base_value = value_from_contents_and_address (baseclass,
-							    buf.data (),
-							    baseclass->length (),
-							    address + boffset);
-	      baseclass = value_type (base_value);
+	      address = address + boffset;
+	      thisoffset = 0;
 	      boffset = 0;
+	      do_cleanups (back_to);
 	    }
+	  else
+	    base_valaddr = valaddr;
 	}
 
-      if (options->prettyformat)
+      if (options->pretty)
 	{
-	  gdb_printf (stream, "\n");
-	  print_spaces (2 * recurse, stream);
+	  fprintf_filtered (stream, "\n");
+	  print_spaces_filtered (2 * recurse, stream);
 	}
-      gdb_puts ("<", stream);
+      fputs_filtered ("<", stream);
       /* Not sure what the best notation is in the case where there is no
-	 baseclass name.  */
+         baseclass name.  */
 
-      gdb_puts (basename ? basename : "", stream);
-      gdb_puts ("> = ", stream);
+      fputs_filtered (basename ? basename : "", stream);
+      fputs_filtered ("> = ", stream);
 
       if (skip < 0)
 	val_print_unavailable (stream);
       else if (skip > 0)
 	val_print_invalid_address (stream);
       else
-	pascal_object_print_value_fields
-	  (base_value, stream, recurse, options,
-	   (struct type **) obstack_base (&dont_print_vb_obstack),
-	   0);
-      gdb_puts (", ", stream);
+	pascal_object_print_value_fields (baseclass, base_valaddr,
+					  thisoffset + boffset, address,
+					  stream, recurse, val, options,
+		     (struct type **) obstack_base (&dont_print_vb_obstack),
+					  0);
+      fputs_filtered (", ", stream);
 
     flush_it:
       ;
@@ -796,10 +823,10 @@ pascal_object_print_value (struct value *val, struct ui_file *stream,
   if (dont_print_vb == 0)
     {
       /* Free the space used to deal with the printing
-	 of this type from top level.  */
+         of this type from top level.  */
       obstack_free (&dont_print_vb_obstack, last_dont_print);
       /* Reset watermark so that we can continue protecting
-	 ourselves from whatever we were protecting ourselves.  */
+         ourselves from whatever we were protecting ourselves.  */
       dont_print_vb_obstack = tmp_obstack;
     }
 }
@@ -822,13 +849,7 @@ pascal_object_print_static_field (struct value *val,
   struct type *type = value_type (val);
   struct value_print_options opts;
 
-  if (value_entirely_optimized_out (val))
-    {
-      val_print_optimized_out (val, stream);
-      return;
-    }
-
-  if (type->code () == TYPE_CODE_STRUCT)
+  if (TYPE_CODE (type) == TYPE_CODE_STRUCT)
     {
       CORE_ADDR *first_dont_print, addr;
       int i;
@@ -842,9 +863,9 @@ pascal_object_print_static_field (struct value *val,
 	{
 	  if (value_address (val) == first_dont_print[i])
 	    {
-	      fputs_styled (_("\
-<same as static member of an already seen type>"),
-			    metadata_style.style (), stream);
+	      fputs_filtered ("\
+<same as static member of an already seen type>",
+			      stream);
 	      return;
 	    }
 	}
@@ -853,9 +874,13 @@ pascal_object_print_static_field (struct value *val,
       obstack_grow (&dont_print_statmem_obstack, (char *) &addr,
 		    sizeof (CORE_ADDR));
 
-      type = check_typedef (type);
-      pascal_object_print_value_fields (val, stream, recurse,
-					options, NULL, 1);
+      CHECK_TYPEDEF (type);
+      pascal_object_print_value_fields (type,
+					value_contents_for_printing (val),
+					value_embedded_offset (val),
+					addr,
+					stream, recurse,
+					val, options, NULL, 1);
       return;
     }
 
@@ -864,9 +889,11 @@ pascal_object_print_static_field (struct value *val,
   common_val_print (val, stream, recurse, &opts, current_language);
 }
 
-void _initialize_pascal_valprint ();
+/* -Wmissing-prototypes */
+extern initialize_file_ftype _initialize_pascal_valprint;
+
 void
-_initialize_pascal_valprint ()
+_initialize_pascal_valprint (void)
 {
   add_setshow_boolean_cmd ("pascal_static-members", class_support,
 			   &user_print_options.pascal_static_field_print, _("\
