@@ -1,5 +1,5 @@
 ! Modified by Linaro.
-! Copyright (C) March 2023-2025 Linaro Limited (or its affiliates). All rights reserved.
+! Copyright (C) March 2023- Linaro Limited (or its affiliates). All rights reserved.
 ! 
 ! Modified by Arm.
 ! Copyright (C) 1995-2023 Arm Limited (or its affiliates). All rights reserved.
@@ -37,11 +37,15 @@ program main
 
   i = 1
   j = 1
+  ! starting break
   allocate (q(1))
-  allocate (q(1)%y(3,5)) !alloc-q-base
-  allocate (p%y(3,5)) !alloc-q-y-contents
+  ! alloc q base
+  allocate (q(1)%y(3,5))
+  ! alloc q y content
+  allocate (p%y(3,5))
+  ! alloc p y content
 
-  do i = 1,3 ! alloc-p-y-content
+  do i = 1,3
      do j = 1,5
         q(1)%x(i,j) = i * 5 + j
         q(1)%y(i,j) = i * 5 + j
@@ -51,7 +55,7 @@ program main
   end do
 
 
-  print *, p%y    ! end-point
-  print *, q(1)%y
+  print *, p%y    ! A great place to stop and grab a beer
+  print *, q(1)%y 
 
 end program main
